import sys
from optparse import OptionParser

app = "APP_NAME"

urlpatterns = [],
default_settings = {
    'ROOT_URLCONF': 'runtests',
    'DATABASES': {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'test.db',
        }
    },
    'INSTALLED_APPS': [
        app,
    ],
}

def runtests(*test_args):
    if not test_args:
        test_args = [app,]

    from django.conf import settings
    settings.configure(**default_settings)

    # Run the test suite, including the extra validation tests.

    from django.test.utils import get_runner
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=False, failfast=False)
    failures = test_runner.run_tests(test_args)
    return failures

def main(*args):
    failures = runtests(*args)
    sys.exit(failures)


if __name__ == "__main__":
    parser = OptionParser()
    (options, args) = parser.parse_args()
    main(*args)



from django.core.management.base import BaseCommand, CommandError
from django.db import connection
from brancher.base import DbNameMixin


class Command(DbNameMixin, BaseCommand):
    help = 'Lists all branches, a -> indicates current. (AKA what branch_it will run against)'

    def handle(self, *args, **options):
        self.change_defaults(**options)
        with connection.cursor() as cursor:
            cursor.execute("SELECT datname FROM pg_database WHERE datname like '{}%';".format(self.get_db_name))
            databases = cursor.fetchall()
            self.stdout.write("Your branched databases:")
            for datname in databases:
                result = datname[0]
                if result == self.full_branched_db_name:
                    self.stdout.write("->\t{}".format(datname[0]))
                else:
                    self.stdout.write("\t{}".format(datname[0]))



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from brancher.base import DbNameMixin
from django.core.management import call_command
from django import db
import argparse


class Command(DbNameMixin, BaseCommand):
    help = 'Run another django command using you current git branch as the branch name'
    _isolate = False
    _migrate = False

    def add_arguments(self, parser):
        parser.add_argument('-i', '--isolate', action='store_true', help="This will run create databse before the command and drop database afterwards")
        parser.add_argument('-m', '--migrate', action='store_true', help="Will run migrate before running the command you specified")
        parser.add_argument('subcommand', nargs=argparse.REMAINDER)
        super(Command, self).add_arguments(parser)

    def change_defaults(self, isolate=False, migrate=False, *args, **options):
        super(Command, self).change_defaults(*args, **options)
        self._isolate = isolate
        self._migrate = migrate

    def handle(self, *args, **options):
        self.change_defaults(**options)
        self.run_if_isolate(self.create_database)
        subcommand = options['subcommand']
        if not subcommand:
            raise CommandError('Please provide command')
        settings.DATABASES['default']['NAME'] = self.full_branched_db_name
        self.migrate_if_needed()
        try:
            call_command(*subcommand)
        except Exception as e:
            self.cleanup_if_needed()
            raise e
        self.cleanup_if_needed()

    def cleanup_if_needed(self):
        settings.DATABASES['default']['NAME'] = self.get_db_name
        db.connections.close_all()
        self.run_if_isolate(self.drop_dataase)

    def run_if_isolate(self, func):
        if self._isolate:
            func()

    def migrate_if_needed(self):
        if self._migrate:
            call_command('migrate')



from django.core.management.base import BaseCommand, CommandError
from brancher.base import DbNameMixin


class Command(DbNameMixin, BaseCommand):
    help = 'Drops a database as long as it is not the default. Defaults to current git branch named database.'

    def handle(self, *args, **options):
        self.change_defaults(**options)
        self.drop_dataase()



from django.conf import settings
from django.db import connection
from subprocess import check_output


class DbNameMixin(object):
    _branch_name = None
    _db_name = None

    def add_arguments(self, parser):
        parser.add_argument('--branch_name', help="This overrides the git branch as the branch name")

    @property
    def full_branched_db_name(self):
        return '{}_{}'.format(self.get_db_name, self.branch_name)

    @property
    def get_db_name(self):
        if self._db_name is None:
            self._db_name = settings.DATABASES['default']['NAME']
        return self._db_name

    @property
    def branch_name(self):
        if self._branch_name is None:
            self._branch_name = check_output('git rev-parse --abbrev-ref HEAD', shell=True).strip()
        return self._branch_name

    def change_defaults(self, branch_name=None, **options):
        self._branch_name = branch_name

    def create_database(self):
        with connection.cursor() as cursor:
            cursor.execute("CREATE DATABASE {branch} template {name};".format(branch=self.full_branched_db_name, name=self.get_db_name))

    def drop_dataase(self):
        with connection.cursor() as cursor:
            cursor.execute("DROP DATABASE IF EXISTS {database};".format(database=self.full_branched_db_name))



from django.core.management.base import BaseCommand, CommandError
from brancher.base import DbNameMixin


class Command(DbNameMixin, BaseCommand):
    help = 'Creates another database branch by default uses your current git branch as the substring and your current default database as the template'

    def handle(self, *args, **options):
        self.change_defaults(**options)
        self.create_database()



"""
setup.py file for building fool components.

Nothing in this file should need to be edited, please see accompanying
package.json file if you need to adjust metadata about this package.

Borrowed almost wholesale from Armstrong http://armstrongcms.org/
"""

from setuptools import setup, find_packages
import json

version = '0.1'

setup_kwargs = {
    "name": "django-brancher",
    "author": "Brian Faherty",
    "author_email": "anothergenericuser@gmail.com",
    "url": "https://github.com/scrummyin/django-brancher",
    "packages": find_packages(),
    "include_package_data": True,
    "version": version,
    "classifiers": [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    "setup_requires": [
        'setuptools-git==1.0'
    ],
}

setup(**setup_kwargs)
